﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.TreeListView;
using Telerik.Windows.Data;
using Telerik.Windows.DragDrop;
using Telerik.Windows.DragDrop.Behaviors;

namespace TreeListViewExample_WPF_NoXAML
{
	public class TreeListViewDragDropBehavior
	{
		private object originalSource = null;
		private IList sourceCollection = null;
		private IList destinationCollection = null;

		private RadTreeListView _associatedObject;

		public IList SourceCollection
		{
			get
			{
				return this.sourceCollection;
			}
			set
			{
				this.sourceCollection = value;
			}
		}

		/// <summary>
		/// AssociatedObject Property
		/// </summary>
		public RadTreeListView AssociatedObject
		{
			get
			{
				return _associatedObject;
			}
			set
			{
				_associatedObject = value;
			}
		}

		private static Dictionary<RadTreeListView, TreeListViewDragDropBehavior> instances;

		static TreeListViewDragDropBehavior()
		{
			instances = new Dictionary<RadTreeListView, TreeListViewDragDropBehavior>();
		}

		public static bool GetIsEnabled(DependencyObject obj)
		{
			return (bool)obj.GetValue(IsEnabledProperty);
		}

		public static void SetIsEnabled(DependencyObject obj, bool value)
		{
			TreeListViewDragDropBehavior behavior = GetAttachedBehavior(obj as RadTreeListView);

			behavior.AssociatedObject = obj as RadTreeListView;

			if (value)
			{
				behavior.Initialize();
			}
			else
			{
				behavior.CleanUp();
			}
			obj.SetValue(IsEnabledProperty, value);
		}

		// Using a DependencyProperty as the backing store for IsEnabled.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty IsEnabledProperty =
			DependencyProperty.RegisterAttached("IsEnabled", typeof(bool), typeof(TreeListViewDragDropBehavior),
				new PropertyMetadata(new PropertyChangedCallback(OnIsEnabledPropertyChanged)));

		public static void OnIsEnabledPropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
		{
#if !SILVERLIGHT
			SetIsEnabled(dependencyObject, (bool)e.NewValue);
#endif
		}

		public static TreeListViewDragDropBehavior GetAttachedBehavior(RadTreeListView treeListView)
		{
			if (!instances.ContainsKey(treeListView))
			{
				instances[treeListView] = new TreeListViewDragDropBehavior();
				instances[treeListView].AssociatedObject = treeListView;
			}

			return instances[treeListView];
		}

		protected virtual void Initialize()
		{
			DragDropManager.AddDragInitializeHandler(this.AssociatedObject, OnDragInitialize, true);
			DragDropManager.AddDropHandler(this.AssociatedObject, OnDrop, true);
			DragDropManager.AddDragDropCompletedHandler(this.AssociatedObject, OnDragDropCompleted, true);
			DragDropManager.AddDragOverHandler(this.AssociatedObject, OnDragOver, true);

			this.AssociatedObject.DataLoaded += RadTreeListView1_DataLoaded;
		}

		protected virtual void CleanUp()
		{
			DragDropManager.RemoveDragInitializeHandler(this.AssociatedObject, OnDragInitialize);
			DragDropManager.RemoveDropHandler(this.AssociatedObject, OnDrop);
			DragDropManager.RemoveDragDropCompletedHandler(this.AssociatedObject, OnDragDropCompleted);
			DragDropManager.RemoveDragOverHandler(this.AssociatedObject, OnDragOver);

			this.AssociatedObject.DataLoaded -= RadTreeListView1_DataLoaded;
		}

		private void OnDragInitialize(object sender, DragInitializeEventArgs e)
		{
			var sourceRow = (e.OriginalSource as TreeListViewRow) ?? (e.OriginalSource as FrameworkElement).ParentOfType<TreeListViewRow>();

			if (sourceRow != null)
			{

				DropIndicationDetails details = new DropIndicationDetails();
				var draggedItem = sourceRow.Item;
				details.CurrentDraggedItem = draggedItem;

				RadObservableCollection<object> selectedItems = new RadObservableCollection<object>();

				foreach (var selectedItem in this.AssociatedObject.SelectedItems)
				{
					selectedItems.Add(selectedItem);
				}

				details.DraggedItems = selectedItems;

				this.sourceCollection = this.AssociatedObject.ItemsSource as RadObservableCollection<WarehouseItem>;
				//foreach (WarehouseItem item in this.AssociatedObject.ItemsSource as ObservableCollection<WarehouseItem>)
				//{
				//	this.sourceCollection.Add(item);
				//}

				IDragPayload dragPayload = DragDropPayloadManager.GeneratePayload(null);

				dragPayload.SetData("DraggedItem", new Collection<object>() { draggedItem });
				dragPayload.SetData("DraggedItems", new Collection<object>() { selectedItems });
				dragPayload.SetData("DropDetails", details);

				e.Data = dragPayload;

				e.DragVisual = new DragVisual()
				{
					Content = details,
					ContentTemplate = this.AssociatedObject.Resources["DraggedItemTemplate"] as DataTemplate
				};
				e.DragVisualOffset = e.RelativeStartPoint;
				e.AllowedEffects = DragDropEffects.All;

			}
		}

		private void OnDragDropCompleted(object sender, DragDropCompletedEventArgs e)
		{
		}

		private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
		{
			if (e.Data != null && e.AllowedEffects != DragDropEffects.None)
			{

				var draggedItem = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItem");
				var droppedItems = (DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedItems") as IList).OfType<RadObservableCollection<object>>().FirstOrDefault();
				var details = DragDropPayloadManager.GetDataFromObject(e.Data, "DropDetails") as DropIndicationDetails;

				if (details == null || draggedItem == null)
				{
					return;
				}

				if (e.Effects == DragDropEffects.Move || e.Effects == DragDropEffects.All)
				{
					var destinationRow = e.OriginalSource as TreeListViewRow ?? (e.OriginalSource as FrameworkElement).ParentOfType<TreeListViewRow>();
					if (destinationRow != null)
					{
						WarehouseItem targetItem = destinationRow.DataContext as WarehouseItem;
						TreeListViewDropPosition relativeDropPosition = (TreeListViewDropPosition)destinationRow.GetValue(RadTreeListView.DropPositionProperty);
						this.destinationCollection = relativeDropPosition == TreeListViewDropPosition.Inside ? (IList)destinationRow.Items.SourceCollection :
																																							 destinationRow.ParentRow != null ? (IList)destinationRow.ParentRow.Items.SourceCollection : (IList)destinationRow.GridViewDataControl.ItemsSource;
						MoveItem(droppedItems, targetItem, relativeDropPosition);
					}
					else
					{
						this.destinationCollection = (IList)(sender as RadTreeListView).ItemsSource;
						MoveItemsToRoot(droppedItems);
					}
				}
			}
		}

		private void OnDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
		{
			var dropTarget = e.OriginalSource as TreeListViewRow ?? (e.OriginalSource as FrameworkElement).ParentOfType<TreeListViewRow>();
			if (this.IsChildOf(dropTarget, this.originalSource))
			{
				e.Effects = DragDropEffects.None;
			}
			
			e.Handled = true;
		}

		public virtual DropPosition GetDropPositionFromPoint(Point absoluteMousePosition, TreeListViewRow row)
		{
			if (row != null)
			{
				return absoluteMousePosition.Y < row.ActualHeight / 2 ? DropPosition.Before : DropPosition.After;
			}

			return DropPosition.Inside;
		}

		private bool IsChildOf(TreeListViewRow dropTarget, object originalSource)
		{
			var currentElement = dropTarget;
			while (currentElement != null)
			{
				if (currentElement.Item == originalSource)
				{
					return true;
				}

				currentElement = currentElement.ParentRow;
			}

			return false;
		}

		void RadTreeListView1_DataLoaded(object sender, EventArgs e)
		{
			this.AssociatedObject.DataLoaded -= new EventHandler<EventArgs>(RadTreeListView1_DataLoaded);
			this.AssociatedObject.ExpandAllHierarchyItems();
		}

		private void MoveItemsToRoot(RadObservableCollection<object> droppedItems)
		{
			var parentCollection = sourceCollection;

			foreach (WarehouseItem droppedItem in droppedItems)
			{
				parentCollection.Remove(droppedItem);
				//droppedItem.ParentFolder = null;
				(destinationCollection).Add(droppedItem);
			}
			

			this.AssociatedObject.ExpandAllHierarchyItems();
		}

		private void RemoveItemsRecursive(RadObservableCollection<WarehouseItem> currentCollection,  RadObservableCollection<object> droppedItems)
		{
			if (currentCollection == null || currentCollection.Count == 0)
			{
				return;
			}

			foreach (WarehouseItem item in currentCollection)
			{
				RemoveItemsRecursive(item.Items, droppedItems);
			}

			foreach (var item in droppedItems)
			{
				if (currentCollection.Contains(item))
				{
					currentCollection.Remove(item as WarehouseItem);
				}

				currentCollection.Reset();
			}
		}

		private void MoveItem(RadObservableCollection<object> droppedItems, WarehouseItem targetItem, TreeListViewDropPosition relativeDropPosition)
		{
			 if (droppedItems.Contains(targetItem))
				return;

			var parentCollection = this.sourceCollection as RadObservableCollection<WarehouseItem>;

			//foreach (WarehouseItem droppedItem in droppedItems)
			//{
			//	parentCollection.Remove(droppedItem);
			//	parentCollection.Reset();
			//}

			RemoveItemsRecursive(parentCollection, droppedItems);
			parentCollection.Reset();

			if (relativeDropPosition == TreeListViewDropPosition.Inside)
			{
				foreach (WarehouseItem droppedItem in droppedItems)
				{
					destinationCollection.Add(droppedItem);
				}
			}
			else if (relativeDropPosition == TreeListViewDropPosition.Before)
			{
				foreach (WarehouseItem droppedItem in droppedItems)
				{
					destinationCollection.Insert(destinationCollection.IndexOf(targetItem), droppedItem);
				}				
			}
			else if (relativeDropPosition == TreeListViewDropPosition.After)
			{
				foreach (WarehouseItem droppedItem in droppedItems)
				{
					destinationCollection.Insert(destinationCollection.IndexOf(targetItem) + 1, droppedItem);
				}				
			}

			this.AssociatedObject.ExpandAllHierarchyItems();
		}
	}
}